﻿using System;
using System.Collections.Generic;

namespace Saml2Code
{
    public class CitizenAttributes
    {
        private static readonly CitizenAttributes instance = new CitizenAttributes();

        private Dictionary<string, Attribute> naturalAttributes;

        private Dictionary<string, Attribute> legalAttributes;

        private Dictionary<string, Attribute> representativeLegalAttributes;

        private Dictionary<string, Attribute> representativeNaturalAttributes;

        private CitizenAttributes()
        {
            PopulatePEPSAttributes();
            return;
        }

        private void PopulatePEPSAttributes()
        {
            char[] attrListSep = ConfigurationSettingsHelper.GetCriticalConfigSetting(CommonConstants.ATTRIBUTE_SEP).ToCharArray();
         
            /* Natural attributes */
            string[] NaturalAttributes = ConfigurationSettingsHelper.GetCriticalConfigSetting(CommonConstants.NaturalAttributes).Split(attrListSep);

            naturalAttributes = new Dictionary<string, Attribute>(NaturalAttributes.Length);
            string now = DateTime.Now.ToString();
            foreach (string attr in NaturalAttributes)
            {
                string attrNS = ConfigurationSettingsHelper.GetCriticalConfigSetting(attr+CommonConstants.ATTRIBUTE_NS_SUFFIX);
                Attribute attribute = new Attribute(attrNS, attr,
                    attr, true, -1, 0, now);
                naturalAttributes.Add(attrNS, attribute);
            }

            /* Representative Legal Attributes*/
            string[] RepresentativeLegalAttributes = ConfigurationSettingsHelper.GetCriticalConfigSetting(CommonConstants.RepresentativeLegalAttributes).Split(attrListSep);

            representativeLegalAttributes = new Dictionary<string, Attribute>(RepresentativeLegalAttributes.Length);
            now = DateTime.Now.ToString();
            foreach (string attr in RepresentativeLegalAttributes) {
                string attrNS = ConfigurationSettingsHelper.GetCriticalConfigSetting(attr + CommonConstants.ATTRIBUTE_NS_SUFFIX);
                Attribute attribute = new Attribute(attrNS, attr,
                    attr, true, -1, 0, now);
                representativeLegalAttributes.Add(attrNS, attribute);
            }

            /* Legal attributes */
            string[] legalAttrList = ConfigurationSettingsHelper.GetCriticalConfigSetting(CommonConstants.LegalAttributes).Split(attrListSep);
            
            legalAttributes = new Dictionary<string, Attribute>(legalAttrList.Length);
            now = DateTime.Now.ToString();
            foreach (string attr in legalAttrList) {
                string attrNS = ConfigurationSettingsHelper.GetCriticalConfigSetting(attr + CommonConstants.ATTRIBUTE_NS_SUFFIX);
                Attribute attribute = new Attribute(attrNS, attr,
                    attr, true, -1, 0, now);
                legalAttributes.Add(attrNS, attribute);
            }

            /* Representative natural attributes */
            string[] RepresentativeNaturalAttributes = ConfigurationSettingsHelper.GetCriticalConfigSetting(CommonConstants.RepresentativeNaturalAttributes).Split(attrListSep);

            representativeNaturalAttributes = new Dictionary<string, Attribute>(RepresentativeNaturalAttributes.Length);
            now = DateTime.Now.ToString();
            foreach (string attr in RepresentativeNaturalAttributes)
            {
                string attrNS = ConfigurationSettingsHelper.GetCriticalConfigSetting(attr + CommonConstants.ATTRIBUTE_NS_SUFFIX);
                Attribute attribute = new Attribute(attrNS, attr,
                    attr, true, -1, 0, now);
                representativeNaturalAttributes.Add(attrNS, attribute);
            }
        }

        public static CitizenAttributes Instance
        {
            get { return instance; }
        }

        public Boolean Exists(string name) {
            if (naturalAttributes.ContainsKey(name) || representativeLegalAttributes.ContainsKey(name) || legalAttributes.ContainsKey(name)) {
                return true;
            } else {
                return false;
            }
        }

        public List<string> GetNaturalNames() {
            return new List<string>(naturalAttributes.Keys);
        }

        public List<string> GetRepresentativeLegalNames() {
            return new List<string>(representativeLegalAttributes.Keys);
        }

        public List<string> GetRepresentativeNames()
        {
            return new List<string>(representativeNaturalAttributes.Keys);

        }
        public List<string> GetLegalNames() {
            return new List<string>(legalAttributes.Keys);
        }

        public string GetFriendlyName(string name)
        {
            if (naturalAttributes.ContainsKey(name))
                return naturalAttributes[name].FriendlyName;
            if (representativeLegalAttributes.ContainsKey(name))
                return representativeLegalAttributes[name].FriendlyName;
            if (legalAttributes.ContainsKey(name))
                return legalAttributes[name].FriendlyName;
            return "none";
        }

        public string GetDescription(string name)
        {
            if (naturalAttributes.ContainsKey(name))
                return naturalAttributes[name].Description;
            if (representativeLegalAttributes.ContainsKey(name))
                return representativeLegalAttributes[name].Description;
            if (legalAttributes.ContainsKey(name))
                return legalAttributes[name].Description;
            if (representativeNaturalAttributes.ContainsKey(name))
                return representativeNaturalAttributes[name].Description;
            return "none";
        }

        public bool HasValueVisualization(string name)
        {
            if (naturalAttributes.ContainsKey(name))
                return naturalAttributes[name].HasValueVisualization;
            if (representativeLegalAttributes.ContainsKey(name))
                return representativeLegalAttributes[name].HasValueVisualization;
            if (legalAttributes.ContainsKey(name))
                return legalAttributes[name].HasValueVisualization;
            return false;
        }

        public int GetProviderId(string name)
        {
            if (naturalAttributes.ContainsKey(name))
                return naturalAttributes[name].AttributeProviderId;
            if (representativeLegalAttributes.ContainsKey(name))
                return representativeLegalAttributes[name].AttributeProviderId;
            if (legalAttributes.ContainsKey(name))
                return legalAttributes[name].AttributeProviderId;
            return -1;
        }

        public int GetOrderIndex(string name)
        {
            if (naturalAttributes.ContainsKey(name))
                return naturalAttributes[name].OrderIndex;
            if (representativeLegalAttributes.ContainsKey(name))
                return representativeLegalAttributes[name].OrderIndex;
            if (legalAttributes.ContainsKey(name))
                return legalAttributes[name].OrderIndex;
            return -1;
        }

        public string GetLastUpdate(string name)
        {
            if (naturalAttributes.ContainsKey(name))
                return naturalAttributes[name].LastUpdate;
            if (representativeLegalAttributes.ContainsKey(name))
                return representativeLegalAttributes[name].LastUpdate;
            if (legalAttributes.ContainsKey(name))
                return legalAttributes[name].LastUpdate;
            return "none";
        }

        private class Attribute
        {
            public Attribute(string name, string friendlyName, string description,
                bool hasValueVisualization, int attrProviderId, int orderIndex, 
                string lastUpdate)
            {
                Name = name;
                FriendlyName = friendlyName;
                Description = description;
                HasValueVisualization = hasValueVisualization;
                AttributeProviderId = attrProviderId;
                OrderIndex = orderIndex;
                LastUpdate = lastUpdate;
            }

            public string Name
            {
                get;
                set;
            }
            public string FriendlyName
            {
                get;
                set;
            }
            public string Description
            {
                get;
                set;
            }
            public bool HasValueVisualization
            {
                get;
                set;
            }
            public int AttributeProviderId
            {
                get;
                set;
            }
            public int OrderIndex
            {
                get;
                set;
            }
            public string LastUpdate
            {
                get;
                set;
            }
            public int SerialNumber
            {
                get;
                set;
            }
        }
    }
}
