﻿using System.Web;
using System.Xml;

namespace Saml2Code
{
    public class Conf
    {
        private static XmlDocument conf;

        /// <summary>
        /// Gets node value from xml
        /// </summary>
        /// <param name="att">node name to be got</param>
        /// <returns>The node value</returns>
        private static string get_data_from_xml(string att)
        {
            if (conf == null)
            {
                conf = new XmlDocument();
                string properties = HttpContext.Current.Server.MapPath("~/App_Data/properties.xml");
                conf.Load(properties);
            }

            XmlNode node = conf.GetElementsByTagName("properties")[0];
            for (int i = 0; i < node.Attributes.Count; i++)
            {
                if (node.Attributes[i].Name == att)
                {
                    return node.Attributes[i].Value;
                }
            }
            return null;
        }

        private static string clock_skew = get_data_from_xml("ClockSkewHours");
        //Error page
        private static string url_fail_return = base_url + "GenericErrorPage.aspx";
        private static string url_return_logout = HttpContext.Current.Request.Url.AbsoluteUri.Substring(0,
            HttpContext.Current.Request.Url.AbsoluteUri.LastIndexOf("/") + 1) + "ResponseLogOut.aspx";
        private static string url_home = base_url + "SP.aspx";
        //This parameters will get the value from properties.xml. It will have URLs,
        //the certs to sign metadata, decrypt, and sign request (dir_p12_signing)
        private static string dir_p12_encryption = get_data_from_xml("dir_p12_encryption");
        private static string cert_password_enc = get_data_from_xml("cert_password_enc");
        private static string dir_p12_signing = get_data_from_xml("dir_p12_signing");
        private static string cert_password_sign = get_data_from_xml("cert_password_sign");
        private static string cert_thumbprint_sign = get_data_from_xml("cert_thumbprint_sign");

        //URL eIDAS service
        private static string url_eidas = get_data_from_xml("url_eidas");

        private static string url_redirect_method = get_data_from_xml("url_redirect_method");
        // Current base URL
        private static string base_url = HttpContext.Current.Request.Url.AbsoluteUri.Substring(0,
            HttpContext.Current.Request.Url.AbsoluteUri.LastIndexOf("/") + 1);
        //URL for asynchronous response
        private static string url_return_response = base_url + "ReturnPage.aspx";
        //Success Page
        private static string url_response_success = base_url + "SuccessPage.aspx";

        //Sign method used in XMLDSig
        private static string sign_method = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512";
        //Digest method used in xml signature
        private static string digest_method = "http://www.w3.org/2001/04/xmlenc#sha512";
        //Canonicalization method used in xml signature
        private static string canonicalization_method = "http://www.w3.org/2001/10/xml-exc-c14n#";
        //Attributes required 
        private static string[] legal_person = new string[] { "http://eidas.europa.eu/attributes/legalperson/LegalName", "http://eidas.europa.eu/attributes/legalperson/LegalPersonIdentifier" };
        private static string[] natural_person = new string[] { "http://eidas.europa.eu/attributes/naturalperson/CurrentFamilyName", "http://eidas.europa.eu/attributes/naturalperson/CurrentGivenName", "http://eidas.europa.eu/attributes/naturalperson/DateOfBirth", "http://eidas.europa.eu/attributes/naturalperson/PersonIdentifier" };
        //path to logs
        private static string dir_log = get_data_from_xml("dir_log");
        //SP provider name
        private static string provider_name = get_data_from_xml("provider_name");

        // WantAssertionSigned Metadata (true or false)
        private static string WantAssertionsSigned = get_data_from_xml("WantAssertionsSigned");
        //date to publish metadata
        private static string issue_instant = Utils.CreateIssueInstant();
        private static string valid_until = Utils.CreateValidUntil();
        /// <summary>
        /// Array values("true" or "False"): ForceAuthn,IsPassive,AllowCreate,D-2012-17-EUIdentifier,EORI,LEI,LegalAdditionalAttribute,LegalAddress,
        /// LegalName,LegalPersonIdentifier,SEED,SIC,TaxReference,VATRegistration,AdditionalAttribute,BirthName,
        /// CurrentAddress,FamilyName,FirstName,DateOfBirth,Gender,PersonIdentifier and PlaceOfBirth.
        /// </summary>
        private static string[] saml2_request_attributes = { "true", "false", "true", "false", "false",
        "false","false","false","true","true","false","false","false","false","false","false","false","true","true","true",
        "false","true","false"};
        //LoA values. The first value will be the default value in default request
        private static string[] LoAForms = {
            "http://eidas.europa.eu/LoA/low",
            "http://eidas.europa.eu/LoA/substantial",
            "http://eidas.europa.eu/LoA/high" };

        /********************************************************************/
        /// <summary>
        /// GETS AND SETS METHODS
        /// </summary>
        /********************************************************************/

        public static string get_url_home() { return url_home; }

        public static string get_url_return_logout() { return url_return_logout; }

        public static string get_url_eidas() { return url_eidas; }

        public static string get_url_fail_return() { return url_fail_return; }

        public static string get_url_response_success() { return url_response_success; }

        public static string[] get_loa_forms() { return LoAForms; }

        public static string get_provider_name() { return provider_name; }

        public static string[] get_saml2_request_attributes() { return saml2_request_attributes; }

        public static string get_url_return_response() { return url_return_response; }

        public static string get_issue_instant()
        {
            return issue_instant;
        }

        public static string get_valid_until()
        {
            return valid_until;
        }

        public static string get_cert_password_sign()
        {
            return cert_password_sign;
        }

        public static string get_cert_password_enc()
        {
            return cert_password_enc;
        }

        public static string get_dir_p12_encryption()
        {
            return dir_p12_encryption;
        }

        public static string get_dir_p12_signing()
        {
            return dir_p12_signing;
        }

        public static string get_cert_thumbprint_sign()
        {
            return cert_thumbprint_sign;
        }

        public static string get_clock_skew()
        {
            return clock_skew;
        }

        public static string get_WantAssertionsSigned()
        {
            return WantAssertionsSigned;
        }


        public static string get_dir_log()
        {
            return dir_log;
        }

        public static string[] get_legal_person()
        {
            return legal_person;
        }

        public static string[] get_natural_person()
        {
            return natural_person;
        }

        public static string get_sign_method()
        {
            return sign_method;
        }

        public static string get_digest_method()
        {
            return digest_method;
        }

        public static string get_canonicalization_method()
        {
            return canonicalization_method;
        }


        public static string get_url_redirect_method()
        {
            return url_redirect_method;
        }


    }
}