﻿using System;
using System.Security.Cryptography.X509Certificates;

namespace Saml2Code
{
    public class Keys
    {
        /// <summary>
        /// Saves a public key in the X509KeyStore.
        /// </summary>
        /// <param name="cert">Public key that must be saved in keystore.</param>
        public static void SavePublicKey(X509Certificate2 cert)
        {
            try
            {
                X509Store store = new X509Store(StoreName.TrustedPeople, StoreLocation.CurrentUser);
                store.Open(OpenFlags.MaxAllowed);
                store.Add(cert);
                store.Close();
            }
            catch (Exception e)
            {
                //LOG: " Problem saving the cert. Please check the cert is the 
                //public key and the keystore name is correct: " + e
            }
        }

        /// <summary>
        /// Checks if public key is in the X509KeyStore with
        /// issuername and serialnumber.
        /// </summary>
        /// <param name="cert">Public key that you want to save in keystore.</param>
        /// <returns>If cert is in KeyStore</returns>
        public static bool IsCertTrustworthy(X509Certificate2 cert)
        {
            try
            {
                X509Store store = new X509Store(StoreName.TrustedPeople, StoreLocation.CurrentUser);

                store.Open(OpenFlags.ReadOnly);

                foreach (X509Certificate2 mCert in store.Certificates)
                {
                    if (mCert.IssuerName.Name == cert.IssuerName.Name &&
                        mCert.SerialNumber == cert.SerialNumber)
                    {
                        //Public key is trustworthy
                        store.Close();
                        return true;
                    }
                }

                store.Close();
            }
            catch (Exception e)
            {
                //LOG: "Problem checking if the cert is in the keystore. 
                //Please check the keystore name and cert is the public key: " + e);
            }

            return false;
        }
    }
}
