﻿using System;
using System.Xml;

namespace Saml2Code
{
    public class XMLData
    {
        /// <summary>
        /// Creates XML to send XML SAML2 to be Loged out (needs to be signed)
        /// </summary>
        /// <returns>XML with SAML2 protocol</returns>
        public static XmlDocument saml2LogOut()
        {
            XmlDocument doc = new XmlDocument();
            doc.PreserveWhitespace = false;
            try
            {
                // logoutrequest/BEGIN
                XmlNode logoutrequest = doc.CreateElement("saml2p", "LogoutRequest", "urn:oasis:names:tc:SAML:2.0:protocol");

                XmlAttribute attribute0 = doc.CreateAttribute("xmlns", "saml2p", "http://www.w3.org/2000/xmlns/");
                attribute0.Value = "urn:oasis:names:tc:SAML:2.0:protocol";

                XmlAttribute attribute1 = doc.CreateAttribute("xmlns", "ds", "http://www.w3.org/2000/xmlns/");
                attribute1.Value = "http://www.w3.org/2000/09/xmldsig#";

                XmlAttribute attribute2 = doc.CreateAttribute("xmlns", "eidas", "http://www.w3.org/2000/xmlns/");
                attribute2.Value = "http://eidas.europa.eu/saml-extensions";

                XmlAttribute attribute3 = doc.CreateAttribute("xmlns", "saml2", "http://www.w3.org/2000/xmlns/");
                attribute3.Value = "urn:oasis:names:tc:SAML:2.0:assertion";

                XmlAttribute Destination = doc.CreateAttribute("Destination");
                Destination.Value = Conf.get_url_eidas();

                XmlAttribute ID = doc.CreateAttribute("ID");
                ID.Value = "_" + Guid.NewGuid().ToString();

                XmlAttribute IssueInstant = doc.CreateAttribute("IssueInstant");
                IssueInstant.Value = Conf.get_issue_instant();

                XmlAttribute Reason = doc.CreateAttribute("Reason");
                Reason.Value = "urn:oasis:names:tc:SAML:2.0:logout:user";

                XmlAttribute Version = doc.CreateAttribute("Version");
                Version.Value = "2.0";

                logoutrequest.Attributes.Append(attribute0);
                logoutrequest.Attributes.Append(attribute1);
                logoutrequest.Attributes.Append(attribute2);
                logoutrequest.Attributes.Append(attribute3);
                logoutrequest.Attributes.Append(Destination);
                logoutrequest.Attributes.Append(ID);
                logoutrequest.Attributes.Append(IssueInstant);
                logoutrequest.Attributes.Append(Reason);
                logoutrequest.Attributes.Append(Version);
                //logoutrequest/END

                // issuer/BEGIN
                XmlNode issuer = doc.CreateElement("saml2", "Issuer", "urn:oasis:names:tc:SAML:2.0:assertion");
                issuer.AppendChild(doc.CreateTextNode(Conf.get_url_return_logout()));
                logoutrequest.AppendChild(issuer);
                //issuer/END

                // nameid/BEGIN
                XmlNode nameid = doc.CreateElement("saml2", "NameID", "urn:oasis:names:tc:SAML:2.0:assertion");

                XmlAttribute Format = doc.CreateAttribute("Format");
                Format.Value = "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified";

                XmlAttribute SPNameQualifier = doc.CreateAttribute("SPNameQualifier");
                SPNameQualifier.Value = Conf.get_url_return_response();

                nameid.Attributes.Append(Format);
                nameid.Attributes.Append(SPNameQualifier);

                nameid.AppendChild(doc.CreateTextNode(Conf.get_provider_name()));

                logoutrequest.AppendChild(nameid);
                // nameid/END

                doc.AppendChild(logoutrequest);
                return doc;
            }
            catch (Exception e)
            {
                //TODO
            }
            return null;
        }

        /// <summary>
        /// Creates XML with protocol SAML2. 
        /// </summary>
        /// <param name="values"></param>
        /// <param name="urlService"></param>
        /// <param name="afirma"></param>
        /// <param name="clavePermanente"></param>
        /// <param name="pin24h"></param>
        /// <param name="eidas"></param>
        /// <param name="idpMovil"></param>
        /// <param name="auth"></param>
        /// <param name="relaystate"></param>
        /// <param name="newLOA"></param>
        /// <returns>XML to send (needs to be signed)</returns>
        public static XmlDocument Saml2RequestDefault(string[] values, string urlService, Boolean afirma, Boolean clavePermanente,
            Boolean pin24h, Boolean eidas, Boolean idpMovil, Boolean auth, Boolean relaystate, string newLOA)
        {
            XmlDocument doc = new XmlDocument();
            doc.PreserveWhitespace = false;

            try
            {
                XmlNode docNode = doc.CreateXmlDeclaration("1.0", "UTF-8", null);
                doc.AppendChild(docNode);
                XmlNode samlreq = doc.CreateElement("saml2p", "AuthnRequest", "urn:oasis:names:tc:SAML:2.0:protocol");

                XmlAttribute destination = doc.CreateAttribute("Destination");
                destination.Value = urlService;
                samlreq.Attributes.Append(destination);

                XmlAttribute eidasNatural = doc.CreateAttribute("xmlns", "eidas-natural", "http://www.w3.org/2000/xmlns/");
                eidasNatural.Value = "http://eidas.europa.eu/attributes/naturalperson";
                samlreq.Attributes.Append(eidasNatural);
                //Adding ProviderName
                XmlAttribute providerName = doc.CreateAttribute("ProviderName");
                providerName.Value = Conf.get_provider_name();
                samlreq.Attributes.Append(providerName);
                //Adding a random ID
                XmlAttribute id = doc.CreateAttribute("ID");
                id.Value = "_" + Guid.NewGuid().ToString();
                samlreq.Attributes.Append(id);
                //Adding the hour and date
                XmlAttribute issueInstant = doc.CreateAttribute("IssueInstant");
                issueInstant.Value = Utils.GetDate();

                samlreq.Attributes.Append(issueInstant);
                //Adding SP return URL
                XmlAttribute assertionConsumerURL = doc.CreateAttribute("AssertionConsumerServiceURL");
                assertionConsumerURL.Value = Conf.get_url_return_response();
                samlreq.Attributes.Append(assertionConsumerURL);
                //xmlns:eidas-natural="http://eidas.europa.eu/attributes/naturalperson"

                XmlAttribute attribute0 = doc.CreateAttribute("xmlns", "saml2p", "http://www.w3.org/2000/xmlns/");
                attribute0.Value = "urn:oasis:names:tc:SAML:2.0:protocol";

                XmlAttribute attribute1 = doc.CreateAttribute("xmlns", "ds", "http://www.w3.org/2000/xmlns/");
                attribute1.Value = "http://www.w3.org/2000/09/xmldsig#";

                XmlAttribute attribute2 = doc.CreateAttribute("xmlns", "eidas", "http://www.w3.org/2000/xmlns/");
                attribute2.Value = "http://eidas.europa.eu/saml-extensions";

                XmlAttribute attribute3 = doc.CreateAttribute("xmlns", "saml2", "http://www.w3.org/2000/xmlns/");
                attribute3.Value = "urn:oasis:names:tc:SAML:2.0:assertion";

                XmlAttribute attribute4 = doc.CreateAttribute("Consent");
                attribute4.Value = "urn:oasis:names:tc:SAML:2.0:consent:unspecified";

                XmlAttribute attribute6 = doc.CreateAttribute("ForceAuthn");
                if (auth) attribute6.Value = "true";
                else attribute6.Value = values[0];

                XmlAttribute attribute8 = doc.CreateAttribute("IsPassive");
                attribute8.Value = values[1];

                XmlAttribute attribute11 = doc.CreateAttribute("Version");
                attribute11.Value = "2.0";

                samlreq.Attributes.Append(attribute0);
                samlreq.Attributes.Append(attribute1);
                samlreq.Attributes.Append(attribute2);
                samlreq.Attributes.Append(attribute3);
                samlreq.Attributes.Append(attribute4);
                samlreq.Attributes.Append(attribute6);
                samlreq.Attributes.Append(attribute8);
                samlreq.Attributes.Append(attribute11);

                XmlNode NameIDPolicy = doc.CreateElement("saml2p", "NameIDPolicy", "urn:oasis:names:tc:SAML:2.0:protocol");
                XmlAttribute AllowCreate = doc.CreateAttribute("AllowCreate");
                AllowCreate.Value = values[2];
                NameIDPolicy.Attributes.Append(AllowCreate);
                XmlAttribute Format = doc.CreateAttribute("Format");
                Format.Value = "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified";
                NameIDPolicy.Attributes.Append(Format);

                XmlNode extensions = doc.CreateElement("saml2p", "Extensions", "urn:oasis:names:tc:SAML:2.0:protocol");

                XmlNode sptype = doc.CreateElement("eidas", "SPType", "http://eidas.europa.eu/saml-extensions");
                sptype.AppendChild(doc.CreateTextNode("public"));
                extensions.AppendChild(sptype);

                XmlNode req = doc.CreateElement("eidas", "RequestedAttributes", "http://eidas.europa.eu/saml-extensions");
                extensions.AppendChild(req);

                if (afirma)
                {
                    XmlNode re0 = doc.CreateElement("eidas", "RequestedAttribute", "http://eidas.europa.eu/saml-extensions");
                    XmlAttribute atre00 = doc.CreateAttribute("FriendlyName");
                    atre00.Value = "AFirmaIdP";

                    XmlAttribute atre01 = doc.CreateAttribute("Name");
                    atre01.Value = "http://es.minhafp.clave/AFirmaIdP";

                    XmlAttribute atre02 = doc.CreateAttribute("NameFormat");
                    atre02.Value = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri";

                    XmlAttribute atre03 = doc.CreateAttribute("isRequired");
                    atre03.Value = "false";

                    re0.Attributes.Append(atre00);
                    re0.Attributes.Append(atre01);
                    re0.Attributes.Append(atre02);
                    re0.Attributes.Append(atre03);

                    req.AppendChild(re0);
                }
                if (clavePermanente)
                {
                    XmlNode re1 = doc.CreateElement("eidas", "RequestedAttribute", "http://eidas.europa.eu/saml-extensions");
                    XmlAttribute atre10 = doc.CreateAttribute("FriendlyName");
                    atre10.Value = "GISSIdP";

                    XmlAttribute atre11 = doc.CreateAttribute("Name");
                    atre11.Value = "http://es.minhafp.clave/GISSIdP";

                    XmlAttribute atre12 = doc.CreateAttribute("NameFormat");
                    atre12.Value = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri";

                    XmlAttribute atre13 = doc.CreateAttribute("isRequired");
                    atre13.Value = "false";

                    re1.Attributes.Append(atre10);
                    re1.Attributes.Append(atre11);
                    re1.Attributes.Append(atre12);
                    re1.Attributes.Append(atre13);

                    req.AppendChild(re1);
                }
                if (pin24h)
                {
                    XmlNode re2 = doc.CreateElement("eidas", "RequestedAttribute", "http://eidas.europa.eu/saml-extensions");
                    XmlAttribute atre20 = doc.CreateAttribute("FriendlyName");
                    atre20.Value = "AEATIdP";

                    XmlAttribute atre21 = doc.CreateAttribute("Name");
                    atre21.Value = "http://es.minhafp.clave/AEATIdP";

                    XmlAttribute atre22 = doc.CreateAttribute("NameFormat");
                    atre22.Value = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri";

                    XmlAttribute atre23 = doc.CreateAttribute("isRequired");
                    atre23.Value = "false";

                    re2.Attributes.Append(atre20);
                    re2.Attributes.Append(atre21);
                    re2.Attributes.Append(atre22);
                    re2.Attributes.Append(atre23);

                    req.AppendChild(re2);
                }
                if (eidas)
                {
                    XmlNode re3 = doc.CreateElement("eidas", "RequestedAttribute", "http://eidas.europa.eu/saml-extensions");
                    XmlAttribute atre30 = doc.CreateAttribute("FriendlyName");
                    atre30.Value = "EIDASIdP";

                    XmlAttribute atre31 = doc.CreateAttribute("Name");
                    atre31.Value = "http://es.minhafp.clave/EIDASIdP";

                    XmlAttribute atre32 = doc.CreateAttribute("NameFormat");
                    atre32.Value = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri";

                    XmlAttribute atre33 = doc.CreateAttribute("isRequired");
                    atre33.Value = "false";

                    re3.Attributes.Append(atre30);
                    re3.Attributes.Append(atre31);
                    re3.Attributes.Append(atre32);
                    re3.Attributes.Append(atre33);

                    req.AppendChild(re3);
                }
                if (relaystate)
                {
                    XmlNode re4 = doc.CreateElement("eidas", "RequestedAttribute", "http://eidas.europa.eu/saml-extensions");

                    XmlAttribute atre40 = doc.CreateAttribute("FriendlyName");
                    atre40.Value = "RelayState";

                    XmlAttribute atre41 = doc.CreateAttribute("Name");
                    atre41.Value = "http://es.minhafp.clave/RelayState";

                    XmlAttribute atre42 = doc.CreateAttribute("NameFormat");
                    atre42.Value = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri";

                    XmlAttribute atre43 = doc.CreateAttribute("isRequired");
                    atre43.Value = "false";

                    XmlNode auxNode = doc.CreateElement("eidas", "AttributeValue", "http://eidas.europa.eu/saml-extensions");
                    XmlAttribute auxAtt1 = doc.CreateAttribute("xmlns", "xsi", "http://www.w3.org/2000/xmlns/");
                    auxAtt1.Value = "http://www.w3.org/2001/XMLSchema-instance";
                    XmlAttribute auxAtt2 = doc.CreateAttribute("xsi", "type", "http://www.w3.org/2001/XMLSchema-instance");
                    auxAtt2.Value = "eidas-natural:PersonIdentifierType";

                    auxNode.Attributes.Append(auxAtt1);
                    auxNode.Attributes.Append(auxAtt2);

                    auxNode.AppendChild(doc.CreateTextNode(Guid.NewGuid().ToString().Substring(0, 8)));

                    re4.Attributes.Append(atre40);
                    re4.Attributes.Append(atre41);
                    re4.Attributes.Append(atre42);
                    re4.Attributes.Append(atre43);

                    re4.AppendChild(auxNode);

                    req.AppendChild(re4);
                }
                if (idpMovil)
                {
                    XmlNode re5 = doc.CreateElement("idpMovil", "RequestedAttribute", "http://eidas.europa.eu/saml-extensions");
                    XmlAttribute atre50 = doc.CreateAttribute("FriendlyName");
                    atre50.Value = "CLVMOVILIdP";

                    XmlAttribute atre51 = doc.CreateAttribute("Name");
                    atre51.Value = "http://es.minhafp.clave/CLVMOVILIdP";

                    XmlAttribute atre52 = doc.CreateAttribute("NameFormat");
                    atre52.Value = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri";

                    XmlAttribute atre53 = doc.CreateAttribute("isRequired");
                    atre53.Value = "false";

                    re5.Attributes.Append(atre50);
                    re5.Attributes.Append(atre51);
                    re5.Attributes.Append(atre52);
                    re5.Attributes.Append(atre53);

                    req.AppendChild(re5);
                }

                XmlNode requestAuthnContext = doc.CreateElement("saml2p", "RequestedAuthnContext", "urn:oasis:names:tc:SAML:2.0:protocol");
                XmlAttribute comparison = doc.CreateAttribute("Comparison");
                comparison.Value = "minimum";
                requestAuthnContext.Attributes.Append(comparison);

                XmlNode authcontextclassref = doc.CreateElement("saml2", "AuthnContextClassRef", "urn:oasis:names:tc:SAML:2.0:assertion");
                //Adding LoA
                if(newLOA != null) authcontextclassref.AppendChild(doc.CreateTextNode(newLOA));
                else authcontextclassref.AppendChild(doc.CreateTextNode(Conf.get_loa_forms()[0]));

                requestAuthnContext.AppendChild(authcontextclassref);

                samlreq.AppendChild(extensions);
                samlreq.AppendChild(NameIDPolicy);
                samlreq.AppendChild(requestAuthnContext);
                //END
                doc.AppendChild(samlreq);
            }
            catch (Exception e)
            {
                //Log: "Error creating the request. Any Conf data could be null: " + e
            }

            return doc;
        }

        /// <summary>
        /// Reads a public key from xml
        /// </summary>
        /// <param name="xml_not_verified">xml</param>
        /// <param name="node">node with public key</param>
        /// <param name="toResponse">if it is a respnse</param>
        /// <returns>The public key</returns>
        public static string ReadPublicKey(XmlDocument xml_not_verified, string node, bool toResponse)
        {
            string publicKey = "";

            try
            {
                XmlNamespaceManager nms1 = new XmlNamespaceManager(xml_not_verified.NameTable);
                if (!toResponse) nms1.AddNamespace("md", "urn:oasis:names:tc:SAML:2.0:metadata");
                nms1.AddNamespace("ds", "http://www.w3.org/2000/09/xmldsig#");
                if (toResponse) nms1.AddNamespace("saml2p", "urn:oasis:names:tc:SAML:2.0:protocol");

                publicKey = xml_not_verified.DocumentElement.SelectSingleNode(node, nms1).InnerXml;
            }
            catch (Exception e)
            {
                //LOG " Error reading the public key: " + e);
            }

            return publicKey;
        }

        /// <summary>
        /// Gets attributes from xml
        /// </summary>
        /// <param name="doc">xml</param>
        /// <returns>attributes array</returns>
        public static string[] GetAttribute(XmlDocument doc)
        {
            var attribute = doc.GetElementsByTagName("saml2:Attribute");

            string[] data = new string[attribute.Count];

            for (int i = 0; i < attribute.Count; i++)
            {
                for (int j = 0; j < attribute[i].Attributes.Count; j++)
                {
                    if (attribute[i].Attributes[j].Name == "Name")
                    {
                        data[i] = attribute[i].Attributes[j].Value;
                    }
                }
            }

            return data;
        }

        /// <summary>
        /// Checks and algorithm from xml
        /// </summary>
        /// <param name="doc">xml</param>
        /// <returns>If the xml has the algorithm</returns>
        public static bool CheckAlgorithm(XmlDocument doc)
        {
            try
            {
                var alg = doc.GetElementsByTagName("alg:SigningMethod");

                for (int j = 0; j < alg.Count; j++)
                {
                    for (int i = 0; i < alg[j].Attributes.Count; i++)
                    {
                        if (alg[j].Attributes[i].Name == "Algorithm")
                        {
                            if (alg[j].Attributes[i].Value == Conf.get_sign_method())
                            {
                                return true;
                            }
                        }
                    }
                }
            }
            catch (Exception e)
            {
                //LOG  " Error reading the signs algorithms. Check the names of the element and attribute name from XML: " + e
            }

            return false;
        }

        /// <summary>
        /// Gets a true if attributes are supported
        /// </summary>
        /// <param name="xml">xml with attributes</param>
        /// <returns>If attribute is supported</returns>
        public static bool CheckAttributes(XmlDocument xml)
        {
            try
            {
                var datas = GetAttribute(xml);
                var naturalperson = Conf.get_natural_person();
                var legalperson = Conf.get_legal_person();

                for (int i = 0; i < naturalperson.Length; i++)
                {
                    bool naturalp = false;

                    foreach (string data in datas)
                    {
                        if (data == naturalperson[i])
                        {
                            naturalp = true;
                        }
                    }
                    if (naturalp == false)
                    {
                        return false;
                    }
                }

                for (int i = 0; i < legalperson.Length; i++)
                {
                    bool legalp = false;

                    foreach (string data in datas)
                    {
                        if (data == legalperson[i])
                        {
                            legalp = true;
                        }
                    }
                    if (legalp == false)
                    {
                        return false;
                    }
                }
            }
            catch (Exception e)
            {
                //LOG :  " Error checking the attributes: " + e
            }

            return true;
        }

        /// <summary>
        /// Gets url to be redirected 
        /// </summary>
        /// <returns>URL</returns>
        public static string GetUrlRedirect(XmlDocument metadata)
        {
            string url = "";
            try
            {
                var data = metadata.GetElementsByTagName("md:SingleSignOnService");
                for (int i = 0; i < data.Count; i++)
                {
                    for (int j = 0; j < data[i].Attributes.Count; j++)
                    {
                        if (data[i].Attributes[j].Name == "Binding" && data[i].Attributes[j].Value == "urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST")
                        {
                            url = data[i].Attributes[j + 1].Value;
                        }
                    }
                }
            }
            catch (Exception e)
            {
                //LOG  " Error getting the URL: " + e
            }
            return url;
        }
    }
}