﻿using System;
using System.Web;
using System.Web.SessionState;
using System.Xml;
using Saml2Code;
using System.IO;

namespace eu.sp.Handlers
{
    public class ResponseHandler : IHttpHandler, IRequiresSessionState
    {
        public void ProcessRequest(HttpContext context)
        {
            string returnAction = Conf.get_url_fail_return();

            try
            {
                //There is the process to get the response from eIDAS node
                //Getting response from eIDAS node
                string samlResponseField = ConfigurationSettingsHelper.GetCriticalConfigSetting("samlResponseField");
                string responseData = context.Request.Form[samlResponseField];

                byte[] reqDataB64 = Convert.FromBase64String(responseData);
                Stream stream = new MemoryStream(reqDataB64);

                //Convert the response to XML form
                XmlDocument xml = new XmlDocument();
                xml.PreserveWhitespace = true;
                xml.Load(stream);
                //Getting the issuer node
                var issuer = xml.GetElementsByTagName("saml2:Issuer");

                //checks eidasNode metadata and SAML response. true as parameter saves cert in your key store
                if (Saml2Code.Utils.VerifyResponse(xml, false))
                {
                    string status = xml.GetElementsByTagName("saml2p:StatusMessage")[0].InnerText;

                    //checking if request was success in the post method
                    if (status == "urn:oasis:names:tc:SAML:2.0:status:Success")
                    {
                        XmlNodeList assertions = xml.GetElementsByTagName("saml2:Assertion");

                        int clockSkew = Int32.Parse(Conf.get_clock_skew());

                        if (assertions.Count > 0)
                        {
                            XmlNodeList Conditions = xml.GetElementsByTagName("saml2:Conditions");

                            if (Conditions.Count > 0)
                            {
								string notBefore = Conditions[0].Attributes["NotBefore"]?.Value;
								string notAfter = Conditions[0].Attributes["NotOnOrAfter"]?.Value;

								if (!string.IsNullOrEmpty(notBefore) && !string.IsNullOrEmpty(notAfter))
								{
									DateTime dateNotbefore = DateTime.Parse(notBefore).ToUniversalTime();
									DateTime dateNotAfter = DateTime.Parse(notAfter).ToUniversalTime();

									if(DateTime.UtcNow < dateNotbefore.AddHours(-clockSkew)
									   || DateTime.UtcNow > dateNotAfter.AddHours(clockSkew))
									{
										Utils.DoLog(new Exception($"Invalid date range, NotBefore: {notBefore} - NotAfter: {notAfter}"));
									}
								}
                            }
                        }

                        XmlDocument request = (XmlDocument)context.Session["SAMLRequest"];
                        string id = request.FirstChild.Attributes.GetNamedItem("ID").InnerText;
                        string inResponseTo = xml.FirstChild.Attributes.GetNamedItem("InResponseTo").InnerText;
                        if (id != null && inResponseTo != null && id.Equals(inResponseTo))
                        {
                            XmlDocument xml_response = null;
                            if (xml.GetElementsByTagName("saml2:EncryptedAssertion").Count > 0)
                            {
                                var dataDecripted = CipherUtils.Decrypt(xml);
                                xml_response = new XmlDocument();
                                xml_response.LoadXml(dataDecripted);
                            }
                            else
                            {
                                xml_response = xml;
                            }

                            context.Session["SAMLResponse"] = xml_response;
                            context.Session["result"] = "success";
                            //Redirect to the next 
                            returnAction = Conf.get_url_response_success();
                        }
                        else
                        {
                            Utils.DoLog(new Exception("InResponseTo field does not match Request ID"));
                        }
                    }
                    else
                    {
                        Utils.DoLog(new Exception("Is a SAMLFail response: " + status));
                    }
                }
                else
                {
                    Utils.DoLog(new Exception("Response or metadata NOT valid"));
                }
            }
            catch (Exception ex)
            {
                //TODO
                returnAction = Conf.get_url_fail_return();
            }
            context.Response.Redirect(returnAction);
        }

        public bool IsReusable
        {
            get { return false; }
        }
    }
}
